@@ -1,6 +1,6 @@
 /*
 
-Copyright (c) 2003-2016, Arvid Norberg
+Copyright (c) 2003-2018, Arvid Norberg
 All rights reserved.
 
 Redistribution and use in source and binary forms, with or without
@@ -60,39 +60,64 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/config.hpp"
 
-#include "libtorrent/aux_/disable_warnings_push.hpp"
-
 #include <map>
 #include <list>
 #include <string>
+#include <vector>
 #include <stdexcept>
-#include <boost/cstdint.hpp>
-#include <boost/config.hpp>
+#include <cstdint>
 #if TORRENT_USE_IOSTREAM
 #include <iosfwd>
 #endif
 
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
-
 #include "libtorrent/assert.hpp"
 #include "libtorrent/error_code.hpp"
-#include "libtorrent/max.hpp"
+#include "libtorrent/span.hpp"
+#include "libtorrent/string_view.hpp"
+#include "libtorrent/aux_/aligned_union.hpp"
 
-namespace libtorrent
-{
-#ifndef TORRENT_NO_DEPRECATE
+namespace libtorrent {
+
+#if TORRENT_ABI_VERSION == 1
 	struct lazy_entry;
+	// backwards compatibility
+	using type_error = system_error;
 #endif
 	struct bdecode_node;
 
-	// thrown by any accessor function of entry if the accessor
-	// function requires a type different than the actual type
-	// of the entry object.
-	struct type_error : std::runtime_error
-	{
-		// internal
-		type_error(const char* error): std::runtime_error(error) {}
-	};
+namespace aux {
+
+#if (__cplusplus > 201103) || (defined _MSC_VER && _MSC_VER >= 1900)
+		// this enables us to compare a string_view against the std::string that's
+		// held by the std::map
+		// is_transparent was introduced in C++14
+		struct strview_less
+		{
+			using is_transparent = std::true_type;
+			template <typename T1, typename T2>
+			bool operator()(T1 const& rhs, T2 const& lhs) const
+			{ return rhs < lhs; }
+		};
+
+		template<class T> using map_string = std::map<std::string, T, aux::strview_less>;
+#else
+		template<class T>
+		struct map_string : std::map<std::string, T>
+		{
+			using base = std::map<std::string, T>;
+
+			typename base::iterator find(const string_view& key)
+			{
+				return this->base::find(key.to_string());
+			}
+
+			typename base::const_iterator find(const string_view& key) const
+			{
+				return this->base::find(key.to_string());
+			}
+		};
+#endif
+	}
 
 	// The ``entry`` class represents one node in a bencoded hierarchy. It works as a
 	// variant type, it can be either a list, a dictionary (``std::map``), an integer
@@ -104,11 +129,11 @@ namespace libtorrent
 		// the key is always a string. If a generic entry would be allowed
 		// as a key, sorting would become a problem (e.g. to compare a string
 		// to a list). The definition doesn't mention such a limit though.
-		typedef std::map<std::string, entry> dictionary_type;
-		typedef std::string string_type;
-		typedef std::list<entry> list_type;
-		typedef boost::int64_t integer_type;
-		typedef std::vector<char> preformatted_type;
+		using dictionary_type = aux::map_string<entry>;
+		using string_type = std::string;
+		using list_type = std::vector<entry>;
+		using integer_type = std::int64_t;
+		using preformatted_type = std::vector<char>;
 
 		// the types an entry can have
 		enum data_type
@@ -127,18 +152,35 @@ namespace libtorrent
 		// constructors directly from a specific type.
 		// The content of the argument is copied into the
 		// newly constructed entry
-		entry(dictionary_type const&);
-		entry(string_type const&);
-		entry(list_type const&);
-		entry(integer_type const&);
-		entry(preformatted_type const&);
+		entry(dictionary_type); // NOLINT
+		entry(span<char const>); // NOLINT
+		template <typename U, typename Cond = typename std::enable_if<
+			std::is_same<U, entry::string_type>::value
+			|| std::is_same<U, string_view>::value
+			|| std::is_same<U, char const*>::value>::type>
+		entry(U v) // NOLINT
+			: m_type(undefined_t)
+		{
+#if TORRENT_USE_ASSERTS
+			m_type_queried = true;
+#endif
+			new(&data) string_type(std::move(v));
+			m_type = string_t;
+		}
+		entry(list_type); // NOLINT
+		entry(integer_type); // NOLINT
+		entry(preformatted_type); // NOLINT
 
 		// construct an empty entry of the specified type.
 		// see data_type enum.
-		entry(data_type t);
+		entry(data_type t); // NOLINT
 
 		// hidden
 		entry(entry const& e);
+		entry(entry&& e) noexcept;
+
+		// construct from bdecode_node parsed form (see bdecode())
+		entry(bdecode_node const& n); // NOLINT
 
 		// hidden
 		entry();
@@ -146,42 +188,52 @@ namespace libtorrent
 		// hidden
 		~entry();
 
-		// hidden
-		bool operator==(entry const& e) const;
-		bool operator!=(entry const& e) const { return !(*this == e); }
-
 		// copies the structure of the right hand side into this
 		// entry.
-#ifndef TORRENT_NO_DEPRECATE
-		void operator=(lazy_entry const&);
+#if TORRENT_ABI_VERSION == 1
+		entry& operator=(lazy_entry const&) &;
 #endif
-		void operator=(bdecode_node const&);
-		void operator=(entry const&);
-		void operator=(dictionary_type const&);
-		void operator=(string_type const&);
-		void operator=(list_type const&);
-		void operator=(integer_type const&);
-		void operator=(preformatted_type const&);
+		entry& operator=(bdecode_node const&) &;
+		entry& operator=(entry const&) &;
+		entry& operator=(entry&&) & noexcept;
+		entry& operator=(dictionary_type) &;
+		entry& operator=(span<char const>) &;
+		template <typename U, typename Cond = typename std::enable_if<
+			std::is_same<U, entry::string_type>::value
+			|| std::is_same<U, char const*>::value>::type>
+		entry& operator=(U v) &
+		{
+			destruct();
+			new(&data) string_type(std::move(v));
+			m_type = string_t;
+#if TORRENT_USE_ASSERTS
+			m_type_queried = true;
+#endif
+			return *this;
+		}
+		entry& operator=(list_type) &;
+		entry& operator=(integer_type) &;
+		entry& operator=(preformatted_type) &;
 
 		// The ``integer()``, ``string()``, ``list()`` and ``dict()`` functions
 		// are accessors that return the respective type. If the ``entry`` object
 		// isn't of the type you request, the accessor will throw
-		// libtorrent_exception (which derives from ``std::runtime_error``). You
-		// can ask an ``entry`` for its type through the ``type()`` function.
-		// 
+		// system_error. You can ask an ``entry`` for its type through the
+		// ``type()`` function.
+		//
 		// If you want to create an ``entry`` you give it the type you want it to
 		// have in its constructor, and then use one of the non-const accessors
 		// to get a reference which you then can assign the value you want it to
 		// have.
-		// 
+		//
 		// The typical code to get info from a torrent file will then look like
 		// this:
 		//
 		// .. code:: c++
-		// 
+		//
 		// 	entry torrent_file;
 		// 	// ...
-		// 
+		//
 		// 	// throws if this is not a dictionary
 		// 	entry::dictionary_type const& dict = torrent_file.dict();
 		// 	entry::dictionary_type::const_iterator i;
@@ -191,23 +243,23 @@ namespace libtorrent
 		// 		std::string tracker_url = i->second.string();
 		// 		std::cout << tracker_url << "\n";
 		// 	}
-		// 
-		// 
+		//
+		//
 		// The following code is equivalent, but a little bit shorter:
 		//
 		// .. code:: c++
-		// 
+		//
 		// 	entry torrent_file;
 		// 	// ...
-		// 
+		//
 		// 	// throws if this is not a dictionary
 		// 	if (entry* i = torrent_file.find_key("announce"))
 		// 	{
 		// 		std::string tracker_url = i->string();
 		// 		std::cout << tracker_url << "\n";
 		// 	}
-		// 
-		// 
+		//
+		//
 		// To make it easier to extract information from a torrent file, the
 		// class torrent_info exists.
 		integer_type& integer();
@@ -225,7 +277,7 @@ namespace libtorrent
 		void swap(entry& e);
 
 		// All of these functions requires the entry to be a dictionary, if it
-		// isn't they will throw ``libtorrent::type_error``.
+		// isn't they will throw ``system_error``.
 		//
 		// The non-const versions of the ``operator[]`` will return a reference
 		// to either the existing element at the given key or, if there is no
@@ -234,28 +286,22 @@ namespace libtorrent
 		//
 		// The const version of ``operator[]`` will only return a reference to an
 		// existing element at the given key. If the key is not found, it will
-		// throw ``libtorrent::type_error``.
-		entry& operator[](char const* key);
-		entry& operator[](std::string const& key);
-#ifndef BOOST_NO_EXCEPTIONS
-		const entry& operator[](char const* key) const;
-		const entry& operator[](std::string const& key) const;
-#endif
+		// throw ``system_error``.
+		entry& operator[](string_view key);
+		const entry& operator[](string_view key) const;
 
 		// These functions requires the entry to be a dictionary, if it isn't
-		// they will throw ``libtorrent::type_error``.
+		// they will throw ``system_error``.
 		//
 		// They will look for an element at the given key in the dictionary, if
-		// the element cannot be found, they will return 0. If an element with
-		// the given key is found, the return a pointer to it.
-		entry* find_key(char const* key);
-		entry const* find_key(char const* key) const;
-		entry* find_key(std::string const& key);
-		entry const* find_key(std::string const& key) const;
+		// the element cannot be found, they will return nullptr. If an element
+		// with the given key is found, the return a pointer to it.
+		entry* find_key(string_view key);
+		entry const* find_key(string_view key) const;
 
 		// returns a pretty-printed string representation
 		// of the bencoded structure, with JSON-style syntax
-		std::string to_string() const;
+		std::string to_string(bool single_line = false) const;
 
 	protected:
 
@@ -265,50 +311,45 @@ namespace libtorrent
 
 	private:
 
-		void to_string_impl(std::string& out, int indent) const;
-
-#if (defined(_MSC_VER) && _MSC_VER < 1310) || TORRENT_COMPLETE_TYPES_REQUIRED
-		// workaround for msvc-bug.
-		// assumes sizeof(map<string, char>) == sizeof(map<string, entry>)
-		// and sizeof(list<char>) == sizeof(list<entry>)
-		enum { union_size
-			= max5<sizeof(std::list<char>)
-			, sizeof(std::map<std::string, char>)
-			, sizeof(string_type)
-			, sizeof(integer_type)
-			, sizeof(preformatted_type)>::value
-		};
+		aux::aligned_union<1
+#if TORRENT_COMPLETE_TYPES_REQUIRED
+			// for implementations that require complete types, use char and hope
+			// for the best
+			, std::list<char>
+			, std::map<std::string, char>
 #else
-		enum { union_size
-			= max5<sizeof(list_type)
-			, sizeof(dictionary_type)
-			, sizeof(string_type)
-			, sizeof(integer_type)
-			, sizeof(preformatted_type)>::value
-		};
+			, list_type
+			, dictionary_type
 #endif
-		integer_type data[(union_size + sizeof(integer_type) - 1)
-			/ sizeof(integer_type)];
+			, preformatted_type
+			, string_type
+			, integer_type
+		>::type data;
 
 		// the bitfield is used so that the m_type_queried field still fits, so
 		// that the ABI is the same for debug builds and release builds. It
 		// appears to be very hard to match debug builds with debug versions of
 		// libtorrent
-		boost::uint8_t m_type:7;
+		std::uint8_t m_type:7;
 
 	public:
 		// in debug mode this is set to false by bdecode to indicate that the
 		// program has not yet queried the type of this entry, and should not
 		// assume that it has a certain type. This is asserted in the accessor
 		// functions. This does not apply if exceptions are used.
-		mutable boost::uint8_t m_type_queried:1;
+		mutable std::uint8_t m_type_queried:1;
 	};
 
-	namespace detail
-	{
-		TORRENT_EXPORT char const* integer_to_str(char* buf, int size
-			, entry::integer_type val);
-	}
+	// hidden
+	TORRENT_EXPORT bool operator==(entry const& lhs, entry const& rhs);
+	inline bool operator!=(entry const& lhs, entry const& rhs) { return !(lhs == rhs); }
+
+namespace detail {
+
+	// internal
+	TORRENT_EXPORT string_view integer_to_str(span<char> buf
+		, entry::integer_type val);
+}
 
 #if TORRENT_USE_IOSTREAM
 	// prints the bencoded structure to the ostream as a JSON-style structure.
@@ -319,16 +360,6 @@ namespace libtorrent
 	}
 #endif
 
-#ifndef BOOST_NO_EXCEPTIONS
-	// internal
-	TORRENT_NO_RETURN inline void throw_type_error()
-	{
-		throw libtorrent_exception(error_code(errors::invalid_entry_type
-			, get_libtorrent_category()));
-	}
-#endif
-
 }
 
 #endif // TORRENT_ENTRY_HPP_INCLUDED
-
