@@ -31,58 +31,42 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/config.hpp"
-
-#include "libtorrent/aux_/disable_warnings_push.hpp"
-
-#include <ctime>
-#include <algorithm>
-#include <set>
-#include <deque>
-#include <cctype>
-#include <algorithm>
-
-#include <boost/limits.hpp>
-#include <boost/bind.hpp>
-
-#ifdef TORRENT_PROFILE_CALLS
-#include <boost/unordered_map.hpp>
-#endif
-
-#include "libtorrent/aux_/disable_warnings_pop.hpp"
-
 #include "libtorrent/extensions/ut_pex.hpp"
 #include "libtorrent/extensions/ut_metadata.hpp"
 #include "libtorrent/extensions/smart_ban.hpp"
-#include "libtorrent/peer_id.hpp"
-#include "libtorrent/torrent_info.hpp"
-#include "libtorrent/tracker_manager.hpp"
-#include "libtorrent/bencode.hpp"
-#include "libtorrent/hasher.hpp"
-#include "libtorrent/entry.hpp"
 #include "libtorrent/session.hpp"
-#include "libtorrent/session_handle.hpp"
-#include "libtorrent/fingerprint.hpp"
-#include "libtorrent/entry.hpp"
-#include "libtorrent/alert_types.hpp"
-#include "libtorrent/invariant_check.hpp"
-#include "libtorrent/file.hpp"
-#include "libtorrent/bt_peer_connection.hpp"
-#include "libtorrent/ip_filter.hpp"
-#include "libtorrent/socket.hpp"
+#include "libtorrent/extensions.hpp"
 #include "libtorrent/aux_/session_impl.hpp"
 #include "libtorrent/aux_/session_call.hpp"
-#include "libtorrent/kademlia/dht_tracker.hpp"
-#include "libtorrent/natpmp.hpp"
-#include "libtorrent/upnp.hpp"
-#include "libtorrent/magnet_uri.hpp"
-#include "libtorrent/lazy_entry.hpp"
+#include "libtorrent/extensions.hpp" // for add_peer_flags_t
 
-using boost::shared_ptr;
-using boost::weak_ptr;
-using libtorrent::aux::session_impl;
+namespace libtorrent {
+
+#ifndef TORRENT_DISABLE_EXTENSIONS
+	// declared in extensions.hpp
+	// remove this once C++17 is required
+	constexpr feature_flags_t plugin::optimistic_unchoke_feature;
+	constexpr feature_flags_t plugin::tick_feature;
+	constexpr feature_flags_t plugin::dht_request_feature;
+	constexpr feature_flags_t plugin::alert_feature;
+#endif
+
+namespace aux {
+	constexpr torrent_list_index_t session_interface::torrent_state_updates;
+	constexpr torrent_list_index_t session_interface::torrent_want_tick;
+	constexpr torrent_list_index_t session_interface::torrent_want_peers_download;
+	constexpr torrent_list_index_t session_interface::torrent_want_peers_finished;
+	constexpr torrent_list_index_t session_interface::torrent_want_scrape;
+	constexpr torrent_list_index_t session_interface::torrent_downloading_auto_managed;
+	constexpr torrent_list_index_t session_interface::torrent_seeding_auto_managed;
+	constexpr torrent_list_index_t session_interface::torrent_checking_auto_managed;
+}
+
+#ifndef TORRENT_DISABLE_EXTENSIONS
+constexpr add_peer_flags_t torrent_plugin::first_time;
+constexpr add_peer_flags_t torrent_plugin::filtered;
+#endif
 
-namespace libtorrent
-{
 namespace {
 
 #if defined TORRENT_ASIO_DEBUGGING
@@ -91,30 +75,29 @@ namespace {
 		int counter = 0;
 		while (log_async())
 		{
-#if defined TORRENT_WINDOWS || defined TORRENT_CYGWIN
-			Sleep(1000);
-#elif defined TORRENT_BEOS
-			snooze_until(system_time() + 1000000, B_SYSTEM_TIMEBASE);
-#else
-			usleep(1000000);
-#endif
+			std::this_thread::sleep_for(seconds(1));
 			++counter;
-			printf("\x1b[2J\x1b[0;0H\x1b[33m==== Waiting to shut down: %d ==== \x1b[0m\n\n"
+			std::printf("\x1b[2J\x1b[0;0H\x1b[33m==== Waiting to shut down: %d ==== \x1b[0m\n\n"
 				, counter);
 		}
 		async_dec_threads();
 
-		fprintf(stderr, "\n\nEXPECTS NO MORE ASYNC OPS\n\n\n");
+		std::fprintf(stderr, "\n\nEXPECTS NO MORE ASYNC OPS\n\n\n");
 	}
 #endif
 } // anonymous namespace
 
-	TORRENT_EXPORT void min_memory_usage(settings_pack& set)
+	settings_pack min_memory_usage()
 	{
+		settings_pack set;
+#if TORRENT_ABI_VERSION == 1
 		// receive data directly into disk buffers
 		// this yields more system calls to read() and
 		// kqueue(), but saves RAM.
 		set.set_bool(settings_pack::contiguous_recv_buffer, false);
+#endif
+
+		set.set_int(settings_pack::max_peer_recv_buffer_size, 32 * 1024 + 200);
 
 		set.set_int(settings_pack::disk_io_write_mode, settings_pack::disable_os_cache);
 		set.set_int(settings_pack::disk_io_read_mode, settings_pack::disable_os_cache);
@@ -123,7 +106,6 @@ namespace {
 		set.set_int(settings_pack::checking_mem_usage, 2);
 
 		// don't use any extra threads to do SHA-1 hashing
-		set.set_int(settings_pack::network_threads, 0);
 		set.set_int(settings_pack::aio_threads, 1);
 
 		set.set_int(settings_pack::alert_queue_size, 100);
@@ -156,19 +138,13 @@ namespace {
 		// socket until the disk write is complete
 		set.set_int(settings_pack::max_queued_disk_bytes, 1);
 
-		// don't keep track of all upnp devices, keep
-		// the device list small
-		set.set_bool(settings_pack::upnp_ignore_nonrouters, true);
-
 		// never keep more than one 16kB block in
 		// the send buffer
 		set.set_int(settings_pack::send_buffer_watermark, 9);
 
 		// don't use any disk cache
 		set.set_int(settings_pack::cache_size, 0);
-		set.set_int(settings_pack::cache_buffer_chunk_size, 1);
 		set.set_bool(settings_pack::use_read_cache, false);
-		set.set_bool(settings_pack::use_disk_read_ahead, false);
 
 		set.set_bool(settings_pack::close_redundant_connections, true);
 
@@ -187,10 +163,12 @@ namespace {
 		// whole pieces
 		set.set_bool(settings_pack::coalesce_reads, false);
 		set.set_bool(settings_pack::coalesce_writes, false);
+		return set;
 	}
 
-	TORRENT_EXPORT void high_performance_seed(settings_pack& set)
+	settings_pack high_performance_seed()
 	{
+		settings_pack set;
 		// don't throttle TCP, assume there is
 		// plenty of bandwidth
 		set.set_int(settings_pack::mixed_mode_algorithm, settings_pack::prefer_tcp);
@@ -198,6 +176,8 @@ namespace {
 		set.set_int(settings_pack::max_out_request_queue, 1500);
 		set.set_int(settings_pack::max_allowed_in_request_queue, 2000);
 
+		set.set_int(settings_pack::max_peer_recv_buffer_size, 5 * 1024 * 1024);
+
 		// we will probably see a high rate of alerts, make it less
 		// likely to loose alerts
 		set.set_int(settings_pack::alert_queue_size, 10000);
@@ -221,20 +201,14 @@ namespace {
 		// allow lots of peers to try to connect simultaneously
 		set.set_int(settings_pack::listen_queue_size, 3000);
 
-		// unchoke many peers
-		set.set_int(settings_pack::unchoke_slots_limit, 2000);
-
-		// we need more DHT capacity to ping more peers
-		// candidates before trying to connect
-		set.set_int(settings_pack::dht_upload_rate_limit, 20000);
+		// unchoke all peers
+		set.set_int(settings_pack::unchoke_slots_limit, -1);
 
 		// use 1 GB of cache
 		set.set_int(settings_pack::cache_size, 32768 * 2);
 		set.set_bool(settings_pack::use_read_cache, true);
-		set.set_int(settings_pack::cache_buffer_chunk_size, 0);
 		set.set_int(settings_pack::read_cache_line_size, 32);
 		set.set_int(settings_pack::write_cache_line_size, 256);
-		set.set_bool(settings_pack::low_prio_disk, false);
 		// 30 seconds expiration to save cache
 		// space for active pieces
 		set.set_int(settings_pack::cache_expiry, 30);
@@ -262,8 +236,7 @@ namespace {
 
 		set.set_int(settings_pack::max_rejects, 10);
 
-		set.set_int(settings_pack::recv_socket_buffer_size, 1024 * 1024);
-		set.set_int(settings_pack::send_socket_buffer_size, 1024 * 1024);
+		set.set_int(settings_pack::send_not_sent_low_watermark, 524288);
 
 		// don't let connections linger for too long
 		set.set_int(settings_pack::request_timeout, 10);
@@ -294,95 +267,83 @@ namespace {
 		// connect to us if they want to
 		set.set_int(settings_pack::max_failcount, 1);
 
-		// the number of threads to use to call async_write_some
-		// and read_some on peer sockets
-		// this doesn't work. See comment in settings_pack.cpp
-		set.set_int(settings_pack::network_threads, 0);
-
 		// number of disk threads for low level file operations
 		set.set_int(settings_pack::aio_threads, 8);
 
 		set.set_int(settings_pack::checking_mem_usage, 2048);
 
-		// the disk cache performs better with the pool allocator
-		set.set_bool(settings_pack::use_disk_cache_pool, true);
+		return set;
 	}
 
-#ifndef TORRENT_NO_DEPRECATE
-	// this function returns a session_settings object
-	// which will optimize libtorrent for minimum memory
-	// usage, with no consideration of performance.
-	TORRENT_EXPORT session_settings min_memory_usage()
+	session_params read_session_params(bdecode_node const& e, save_state_flags_t const flags)
 	{
-		aux::session_settings def;
-		settings_pack pack;
-		min_memory_usage(pack);
-		apply_pack(&pack, def, 0);
-		session_settings ret;
-		load_struct_from_settings(def, ret);
-		return ret;
+		session_params params;
+
+		bdecode_node settings;
+		if (e.type() != bdecode_node::dict_t) return params;
+
+		if (flags & session_handle::save_settings)
+		{
+			settings = e.dict_find_dict("settings");
+			if (settings)
+			{
+				params.settings = load_pack_from_dict(settings);
+			}
+		}
+
+#ifndef TORRENT_DISABLE_DHT
+		if (flags & session_handle::save_dht_settings)
+		{
+			settings = e.dict_find_dict("dht");
+			if (settings)
+			{
+				params.dht_settings = dht::read_dht_settings(settings);
+			}
+		}
+
+		if (flags & session_handle::save_dht_state)
+		{
+			settings = e.dict_find_dict("dht state");
+			if (settings)
+			{
+				params.dht_state = dht::read_dht_state(settings);
+			}
+		}
+#endif
+
+		return params;
 	}
 
-	TORRENT_EXPORT session_settings high_performance_seed()
+	void session::start(session_params&& params, io_service* ios)
 	{
-		aux::session_settings def;
-		settings_pack pack;
-		high_performance_seed(pack);
-		apply_pack(&pack, def, 0);
-		session_settings ret;
-		load_struct_from_settings(def, ret);
-		return ret;
-	}
-#endif
-
-#define TORRENT_ASYNC_CALL(x) \
-	m_impl->get_io_service().dispatch(boost::bind(&session_impl:: x, m_impl.get()))
-
-#ifndef TORRENT_CFG
-#error TORRENT_CFG is not defined!
-#endif
-
-	// this is a dummy function that's exported and named based
-	// on the configuration. The session.hpp file will reference
-	// it and if the library and the client are built with different
-	// configurations this will give a link error
-	void TORRENT_EXPORT TORRENT_CFG() {}
-
-#if defined _MSC_VER && defined TORRENT_DEBUG
-	static void straight_to_debugger(unsigned int, _EXCEPTION_POINTERS*)
-	{ throw; }
-#endif
-
-	void session::start(int flags, settings_pack const& pack, io_service* ios)
-	{
-#if defined _MSC_VER && defined TORRENT_DEBUG
-		// workaround for microsoft's
-		// hardware exceptions that makes
-		// it hard to debug stuff
-		::_set_se_translator(straight_to_debugger);
-#endif
-
-		bool const internal_executor = ios == NULL;
+		bool const internal_executor = ios == nullptr;
 
 		if (internal_executor)
 		{
 			// the user did not provide an executor, we have to use our own
-			m_io_service = boost::make_shared<io_service>();
+			m_io_service = std::make_shared<io_service>(1);
 			ios = m_io_service.get();
 		}
 
-		m_impl = boost::make_shared<session_impl>(boost::ref(*ios), boost::ref(pack));
-		*static_cast<session_handle*>(this) = session_handle(m_impl.get());
+		m_impl = std::make_shared<aux::session_impl>(std::ref(*ios), std::ref(params.settings));
+		*static_cast<session_handle*>(this) = session_handle(m_impl);
 
 #ifndef TORRENT_DISABLE_EXTENSIONS
-		if (flags & add_default_plugins)
+		for (auto& ext : params.extensions)
 		{
-			add_extension(create_ut_pex_plugin);
-			add_extension(create_ut_metadata_plugin);
-			add_extension(create_smart_ban_plugin);
+			m_impl->add_ses_extension(std::move(ext));
 		}
-#else
-		TORRENT_UNUSED(flags);
+#endif
+
+#ifndef TORRENT_DISABLE_DHT
+		if (params.settings.has_val(settings_pack::dht_upload_rate_limit))
+			params.dht_settings.upload_rate_limit = params.settings.get_int(settings_pack::dht_upload_rate_limit);
+
+		m_impl->set_dht_settings(std::move(params.dht_settings));
+		m_impl->set_dht_state(std::move(params.dht_state));
+
+		TORRENT_ASSERT(params.dht_storage_constructor);
+		m_impl->set_dht_storage(std::move(params.dht_storage_constructor));
 #endif
 
 		m_impl->start_session();
@@ -390,19 +351,47 @@ namespace {
 		if (internal_executor)
 		{
 			// start a thread for the message pump
-			m_thread = boost::make_shared<thread>(boost::bind(&io_service::run
-				, m_io_service.get()));
+			m_thread = std::make_shared<std::thread>(
+				[&] { m_io_service->run(); });
 		}
 	}
 
+namespace {
+
+		std::vector<std::shared_ptr<plugin>> default_plugins(
+			bool empty = false)
+		{
+#ifndef TORRENT_DISABLE_EXTENSIONS
+			if (empty) return {};
+			using wrapper = aux::session_impl::session_plugin_wrapper;
+			return {
+				std::make_shared<wrapper>(create_ut_pex_plugin),
+				std::make_shared<wrapper>(create_ut_metadata_plugin),
+				std::make_shared<wrapper>(create_smart_ban_plugin)
+			};
+#else
+			TORRENT_UNUSED(empty);
+			return {};
+#endif
+		}
+	}
+
+	void session::start(session_flags_t const flags, settings_pack&& sp, io_service* ios)
+	{
+		start({std::move(sp),
+			default_plugins(!(flags & add_default_plugins))}, ios);
+	}
+
 	session::~session()
 	{
 		aux::dump_call_profile();
-
 		TORRENT_ASSERT(m_impl);
-		TORRENT_ASYNC_CALL(abort);
 
-		if (m_thread && m_thread.unique())
+		// capture the shared_ptr in the dispatched function
+		// to keep the session_impl alive
+		m_impl->call_abort();
+
+		if (m_thread && m_thread.use_count() == 1)
 		{
 #if defined TORRENT_ASIO_DEBUGGING
 			wait_for_asio_handlers();
@@ -415,24 +404,25 @@ namespace {
 	{
 		// stop calling the alert notify function now, to avoid it thinking the
 		// session is still alive
-		m_impl->alerts().set_notify_function(boost::function<void()>());
+		m_impl->alerts().set_notify_function({});
 		return session_proxy(m_io_service, m_thread, m_impl);
 	}
 
-#ifndef TORRENT_NO_DEPRECATE
-	session_settings::session_settings(std::string const& user_agent_)
-	{
-		aux::session_settings def;
-		def.set_str(settings_pack::user_agent, user_agent_);
-		load_struct_from_settings(def, *this);
-	}
-
-	session_settings::~session_settings() {}
-#endif // TORRENT_NO_DEPRECATE
-
+	session_proxy::session_proxy() = default;
+	session_proxy::session_proxy(std::shared_ptr<io_service> ios
+		, std::shared_ptr<std::thread> t
+		, std::shared_ptr<aux::session_impl> impl)
+		: m_io_service(std::move(ios))
+		, m_thread(std::move(t))
+		, m_impl(std::move(impl))
+	{}
+	session_proxy::session_proxy(session_proxy const&) = default;
+	session_proxy& session_proxy::operator=(session_proxy const&) = default;
+	session_proxy::session_proxy(session_proxy&&) noexcept = default;
+	session_proxy& session_proxy::operator=(session_proxy&&) noexcept = default;
 	session_proxy::~session_proxy()
 	{
-		if (m_thread && m_thread.unique())
+		if (m_thread && m_thread.use_count() == 1)
 		{
 #if defined TORRENT_ASIO_DEBUGGING
 			wait_for_asio_handlers();
@@ -440,5 +430,37 @@ namespace {
 			m_thread->join();
 		}
 	}
-}
 
+	session_params::session_params(settings_pack&& sp)
+		: session_params(std::move(sp), default_plugins())
+	{}
+
+	session_params::session_params(settings_pack const& sp)
+		: session_params(sp, default_plugins())
+	{}
+
+	session_params::session_params()
+		: extensions(default_plugins())
+#ifndef TORRENT_DISABLE_DHT
+		, dht_storage_constructor(dht::dht_default_storage_constructor)
+#endif
+	{}
+
+	session_params::session_params(settings_pack&& sp
+		, std::vector<std::shared_ptr<plugin>> exts)
+		: settings(std::move(sp))
+		, extensions(std::move(exts))
+#ifndef TORRENT_DISABLE_DHT
+		, dht_storage_constructor(dht::dht_default_storage_constructor)
+#endif
+	{}
+
+	session_params::session_params(settings_pack const& sp
+		, std::vector<std::shared_ptr<plugin>> exts)
+		: settings(sp)
+		, extensions(std::move(exts))
+#ifndef TORRENT_DISABLE_DHT
+		, dht_storage_constructor(dht::dht_default_storage_constructor)
+#endif
+	{}
+}
