@@ -32,13 +32,13 @@ POSSIBILITY OF SUCH DAMAGE.
 
 #include "libtorrent/config.hpp"
 
-// on windows we need these functions for
-// convert_to_native and convert_from_native
-#if TORRENT_USE_WSTRING || defined TORRENT_WINDOWS
-
 #include <iterator>
 #include "libtorrent/utf8.hpp"
+#include "libtorrent/assert.hpp"
+#include "libtorrent/error_code.hpp"
 #include "libtorrent/ConvertUTF.h"
+#include "libtorrent/aux_/throw.hpp"
+#include "libtorrent/aux_/numeric_cast.hpp"
 
 
 #ifdef __clang__
@@ -46,22 +46,22 @@ POSSIBILITY OF SUCH DAMAGE.
 #pragma clang diagnostic ignored "-Wunused-member-function"
 #endif
 
-namespace libtorrent
-{
-	namespace
-	{
+namespace libtorrent {
+
+namespace {
+
 		// ==== utf-8 -> wide ===
 		template<int width>
 		struct convert_to_wide
 		{
-			static utf8_conv_result_t convert(UTF8 const** src_start
+			static utf8_errors::error_code_enum convert(UTF8 const** src_start
 				, UTF8 const* src_end
 				, std::wstring& wide)
 			{
 				TORRENT_UNUSED(src_start);
 				TORRENT_UNUSED(src_end);
 				TORRENT_UNUSED(wide);
-				return source_illegal;
+				return utf8_errors::error_code_enum::source_illegal;
 			}
 		};
 
@@ -69,7 +69,7 @@ namespace libtorrent
 		template<>
 		struct convert_to_wide<4>
 		{
-			static utf8_conv_result_t convert(char const** src_start
+			static utf8_errors::error_code_enum convert(char const** src_start
 				, char const* src_end
 				, std::wstring& wide)
 			{
@@ -84,13 +84,13 @@ namespace libtorrent
 				{
 					// assume Latin-1
 					wide.clear();
-					std::copy(reinterpret_cast<boost::uint8_t const*>(*src_start)
-						,reinterpret_cast<boost::uint8_t const*>(src_end)
+					std::copy(reinterpret_cast<std::uint8_t const*>(*src_start)
+						,reinterpret_cast<std::uint8_t const*>(src_end)
 						, std::back_inserter(wide));
-					return static_cast<utf8_conv_result_t>(ret);
+					return static_cast<utf8_errors::error_code_enum>(ret);
 				}
-				wide.resize(dst_start - wide.c_str());
-				return static_cast<utf8_conv_result_t>(ret);
+				wide.resize(aux::numeric_cast<std::size_t>(dst_start - wide.c_str()));
+				return static_cast<utf8_errors::error_code_enum>(ret);
 			}
 		};
 
@@ -98,7 +98,7 @@ namespace libtorrent
 		template<>
 		struct convert_to_wide<2>
 		{
-			static utf8_conv_result_t convert(char const** src_start
+			static utf8_errors::error_code_enum convert(char const** src_start
 				, char const* src_end
 				, std::wstring& wide)
 			{
@@ -113,13 +113,13 @@ namespace libtorrent
 				{
 					// assume Latin-1
 					wide.clear();
-					std::copy(reinterpret_cast<boost::uint8_t const*>(*src_start)
-						, reinterpret_cast<boost::uint8_t const*>(src_end)
+					std::copy(reinterpret_cast<std::uint8_t const*>(*src_start)
+						, reinterpret_cast<std::uint8_t const*>(src_end)
 						, std::back_inserter(wide));
-					return static_cast<utf8_conv_result_t>(ret);
+					return static_cast<utf8_errors::error_code_enum>(ret);
 				}
-				wide.resize(dst_start - wide.c_str());
-				return static_cast<utf8_conv_result_t>(ret);
+				wide.resize(aux::numeric_cast<std::size_t>(dst_start - wide.c_str()));
+				return static_cast<utf8_errors::error_code_enum>(ret);
 			}
 		};
 
@@ -127,14 +127,14 @@ namespace libtorrent
 		template<int width>
 		struct convert_from_wide
 		{
-			static utf8_conv_result_t convert(wchar_t const** src_start
+			static utf8_errors::error_code_enum convert(wchar_t const** src_start
 				, wchar_t const* src_end
 				, std::string& utf8)
 			{
 				TORRENT_UNUSED(src_start);
 				TORRENT_UNUSED(src_end);
 				TORRENT_UNUSED(utf8);
-				return source_illegal;
+				return utf8_errors::error_code_enum::source_illegal;
 			}
 		};
 
@@ -142,7 +142,7 @@ namespace libtorrent
 		template<>
 		struct convert_from_wide<4>
 		{
-			static utf8_conv_result_t convert(wchar_t const** src_start
+			static utf8_errors::error_code_enum convert(wchar_t const** src_start
 				, wchar_t const* src_end
 				, std::string& utf8)
 			{
@@ -153,8 +153,8 @@ namespace libtorrent
 					, reinterpret_cast<UTF8**>(&dst_start)
 					, reinterpret_cast<UTF8*>(dst_start + utf8.size())
 					, lenientConversion);
-				utf8.resize(dst_start - &utf8[0]);
-				return static_cast<utf8_conv_result_t>(ret);
+				utf8.resize(aux::numeric_cast<std::size_t>(dst_start - &utf8[0]));
+				return static_cast<utf8_errors::error_code_enum>(ret);
 			}
 		};
 
@@ -162,7 +162,7 @@ namespace libtorrent
 		template<>
 		struct convert_from_wide<2>
 		{
-			static utf8_conv_result_t convert(wchar_t const** src_start
+			static utf8_errors::error_code_enum convert(wchar_t const** src_start
 				, wchar_t const* src_end
 				, std::string& utf8)
 			{
@@ -173,53 +173,118 @@ namespace libtorrent
 					, reinterpret_cast<UTF8**>(&dst_start)
 					, reinterpret_cast<UTF8*>(dst_start + utf8.size())
 					, lenientConversion);
-				utf8.resize(dst_start - &utf8[0]);
-				return static_cast<utf8_conv_result_t>(ret);
+				utf8.resize(aux::numeric_cast<std::size_t>(dst_start - &utf8[0]));
+				return static_cast<utf8_errors::error_code_enum>(ret);
 			}
 		};
+
+		struct utf8_error_category final : boost::system::error_category
+		{
+			const char* name() const BOOST_SYSTEM_NOEXCEPT override
+			{
+				return "UTF error";
+			}
+
+			std::string message(int ev) const override
+			{
+				static char const* error_messages[] = {
+					"ok",
+					"source exhausted",
+					"target exhausted",
+					"source illegal"
+				};
+
+				TORRENT_ASSERT(ev >= 0);
+				TORRENT_ASSERT(ev < int(sizeof(error_messages)/sizeof(error_messages[0])));
+				return error_messages[ev];
+			}
+
+			boost::system::error_condition default_error_condition(
+				int ev) const BOOST_SYSTEM_NOEXCEPT override
+			{ return {ev, *this}; }
+		};
 	} // anonymous namespace
 
-	utf8_conv_result_t utf8_wchar(std::string const& utf8, std::wstring &wide)
+	namespace utf8_errors
 	{
-		// allocate space for worst-case
-		wide.resize(utf8.size());
-		char const* src_start = utf8.c_str();
-		return convert_to_wide<sizeof(wchar_t)>::convert(
-			&src_start, src_start + utf8.size(), wide);
+		boost::system::error_code make_error_code(utf8_errors::error_code_enum e)
+		{
+			return {e, utf8_category()};
+		}
+	} // utf_errors namespace
+
+	boost::system::error_category const& utf8_category()
+	{
+		static utf8_error_category cat;
+		return cat;
 	}
 
-	utf8_conv_result_t wchar_utf8(std::wstring const& wide, std::string &utf8)
+	std::wstring utf8_wchar(string_view utf8, error_code& ec)
 	{
 		// allocate space for worst-case
+		std::wstring wide;
+		wide.resize(utf8.size());
+		char const* src_start = utf8.data();
+		utf8_errors::error_code_enum const ret = convert_to_wide<sizeof(wchar_t)>::convert(
+			&src_start, src_start + utf8.size(), wide);
+		if (ret != utf8_errors::error_code_enum::conversion_ok)
+			ec = make_error_code(ret);
+		return wide;
+	}
+
+	std::wstring utf8_wchar(string_view wide)
+	{
+		error_code ec;
+		std::wstring ret = utf8_wchar(wide, ec);
+		if (ec) aux::throw_ex<system_error>(ec);
+		return ret;
+	}
+
+	std::string wchar_utf8(wstring_view wide, error_code& ec)
+	{
+		// allocate space for worst-case
+		std::string utf8;
 		utf8.resize(wide.size() * 6);
-		if (wide.empty()) return conversion_ok;
-		wchar_t const* src_start = wide.c_str();
-		return convert_from_wide<sizeof(wchar_t)>::convert(
+		if (wide.empty()) return {};
+
+		wchar_t const* src_start = wide.data();
+		utf8_errors::error_code_enum const ret = convert_from_wide<sizeof(wchar_t)>::convert(
 			&src_start, src_start + wide.size(), utf8);
+		if (ret != utf8_errors::error_code_enum::conversion_ok)
+			ec = make_error_code(ret);
+		return utf8;
+	}
+
+	std::string wchar_utf8(wstring_view wide)
+	{
+		error_code ec;
+		std::string ret = wchar_utf8(wide, ec);
+		if (ec) aux::throw_ex<system_error>(ec);
+		return ret;
 	}
 
 	// returns the unicode codepoint and the number of bytes of the utf8 sequence
 	// that was parsed. The codepoint is -1 if it's invalid
-	std::pair<boost::int32_t, int> parse_utf8_codepoint(char const* str, int const len)
+	std::pair<std::int32_t, int> parse_utf8_codepoint(string_view str)
 	{
-		int const sequence_len = trailingBytesForUTF8[static_cast<boost::uint8_t>(*str)] + 1;
-		if (sequence_len > len) return std::make_pair(-1, len);
+		int const sequence_len = trailingBytesForUTF8[static_cast<std::uint8_t>(str[0])] + 1;
+		if (sequence_len > int(str.size())) return std::make_pair(-1, static_cast<int>(str.size()));
 
 		if (sequence_len > 4)
 		{
 			return std::make_pair(-1, sequence_len);
 		}
 
-		if (!isLegalUTF8(reinterpret_cast<UTF8 const*>(str), sequence_len))
+		if (!isLegalUTF8(reinterpret_cast<UTF8 const*>(str.data()), sequence_len))
 		{
 			return std::make_pair(-1, sequence_len);
 		}
 
-		boost::uint32_t ch = 0;
+		std::uint32_t ch = 0;
 		for (int i = 0; i < sequence_len; ++i)
 		{
 			ch <<= 6;
-			ch += static_cast<boost::uint8_t>(str[i]);
+			ch += static_cast<std::uint8_t>(str[static_cast<std::size_t>(i)]);
 		}
 		ch -= offsetsFromUTF8[sequence_len-1];
 
@@ -228,7 +293,7 @@ namespace libtorrent
 			return std::make_pair(-1, sequence_len);
 		}
 
-		return std::make_pair(static_cast<boost::int32_t>(ch), sequence_len);
+		return std::make_pair(static_cast<std::int32_t>(ch), sequence_len);
 	}
 }
 
@@ -236,5 +301,3 @@ namespace libtorrent
 #pragma clang diagnostic pop
 #endif
 
-#endif
-
