@@ -31,21 +31,25 @@ POSSIBILITY OF SUCH DAMAGE.
 */
 
 #include "libtorrent/bencode.hpp"
+#include "libtorrent/bdecode.hpp"
 
 #include <iostream>
 #include <cstring>
+#include <utility>
 
-#ifndef TORRENT_NO_DEPRECATE
+#if TORRENT_ABI_VERSION == 1
 #include "libtorrent/lazy_entry.hpp"
 #endif
 
 #include "test.hpp"
 
-using namespace libtorrent;
+using namespace lt;
 
 // test vectors from bittorrent protocol description
 // http://www.bittorrent.com/protocol.html
 
+namespace {
+
 std::string encode(entry const& e)
 {
 	std::string ret;
@@ -53,37 +57,34 @@ std::string encode(entry const& e)
 	return ret;
 }
 
-entry decode(std::string const& str)
-{
-	return bdecode(str.begin(), str.end());
-}
+} // anonymous namespace
 
 TORRENT_TEST(strings)
 {
 	entry e("spam");
 	TEST_CHECK(encode(e) == "4:spam");
-	TEST_CHECK(decode(encode(e)) == e);
+	TEST_CHECK(bdecode(encode(e)) == e);
 }
 
 TORRENT_TEST(integers)
 {
 	entry e(3);
 	TEST_CHECK(encode(e) == "i3e");
-	TEST_CHECK(decode(encode(e)) == e);
+	TEST_CHECK(bdecode(encode(e)) == e);
 }
 
 TORRENT_TEST(integers2)
 {
 	entry e(-3);
 	TEST_CHECK(encode(e) == "i-3e");
-	TEST_CHECK(decode(encode(e)) == e);
+	TEST_CHECK(bdecode(encode(e)) == e);
 }
 
 TORRENT_TEST(integers3)
 {
 	entry e(int(0));
 	TEST_CHECK(encode(e) == "i0e");
-	TEST_CHECK(decode(encode(e)) == e);
+	TEST_CHECK(bdecode(encode(e)) == e);
 }
 
 TORRENT_TEST(lists)
@@ -93,7 +94,7 @@ TORRENT_TEST(lists)
 	l.push_back(entry("eggs"));
 	entry e(l);
 	TEST_CHECK(encode(e) == "l4:spam4:eggse");
-	TEST_CHECK(decode(encode(e)) == e);
+	TEST_CHECK(bdecode(encode(e)) == e);
 }
 
 TORRENT_TEST(dictionaries)
@@ -102,7 +103,7 @@ TORRENT_TEST(dictionaries)
 	e["spam"] = entry("eggs");
 	e["cow"] = entry("moo");
 	TEST_CHECK(encode(e) == "d3:cow3:moo4:spam4:eggse");
-	TEST_CHECK(decode(encode(e)) == e);
+	TEST_CHECK(bdecode(encode(e)) == e);
 }
 
 TORRENT_TEST(preformatted)
@@ -134,6 +135,13 @@ TORRENT_TEST(undefined_node2)
 	TEST_EQUAL(encode(e), "d4:info0:e");
 }
 
+TORRENT_TEST(implicit_construct)
+{
+	entry e(entry::list_t);
+	e.list().push_back(entry::list_t);
+	TEST_EQUAL(e.list().back().type(), entry::list_t);
+}
+
 TORRENT_TEST(print_dict_single_line)
 {
 	entry e;
@@ -215,7 +223,19 @@ TORRENT_TEST(print_deep_dict)
 	TEST_EQUAL(e.to_string(), "{\n 'a': 'foobar',\n 'ints': [\n   1,\n   2,\n   3 ],\n 'strings': [\n   'foo',\n   'bar' ] }");
 }
 
-#ifndef TORRENT_NO_DEPRECATE
+TORRENT_TEST(integer_to_str)
+{
+	using lt::detail::integer_to_str;
+
+	char buf[30];
+	TEST_CHECK(integer_to_str(buf, 0) == "0"_sv);
+	TEST_CHECK(integer_to_str(buf, 1234) == "1234"_sv);
+	TEST_CHECK(integer_to_str(buf, -1234) == "-1234"_sv);
+	TEST_CHECK(integer_to_str(buf, 123456789012345678LL) == "123456789012345678"_sv);
+	TEST_CHECK(integer_to_str(buf, -123456789012345678LL) == "-123456789012345678"_sv);
+}
+
+#if TORRENT_ABI_VERSION == 1
 TORRENT_TEST(lazy_entry)
 {
 	{
@@ -224,9 +244,9 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret == 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 		std::pair<const char*, int> section = e.data_section();
-		TEST_CHECK(std::memcmp(b, section.first, section.second) == 0);
+		TEST_CHECK(std::memcmp(b, section.first, std::size_t(section.second)) == 0);
 		TEST_CHECK(section.second == sizeof(b) - 1);
 		TEST_CHECK(e.type() == lazy_entry::int_t);
 		TEST_CHECK(e.int_value() == 12453);
@@ -238,9 +258,9 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret == 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 		std::pair<const char*, int> section = e.data_section();
-		TEST_CHECK(std::memcmp(b, section.first, section.second) == 0);
+		TEST_CHECK(std::memcmp(b, section.first, std::size_t(section.second)) == 0);
 		TEST_CHECK(section.second == sizeof(b) - 1);
 		TEST_CHECK(e.type() == lazy_entry::string_t);
 		TEST_CHECK(e.string_value() == std::string("abcdefghijklmnopqrstuvwxyz"));
@@ -253,9 +273,9 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret == 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 		std::pair<const char*, int> section = e.data_section();
-		TEST_CHECK(std::memcmp(b, section.first, section.second) == 0);
+		TEST_CHECK(std::memcmp(b, section.first, std::size_t(section.second)) == 0);
 		TEST_CHECK(section.second == sizeof(b) - 1);
 		TEST_CHECK(e.type() == lazy_entry::list_t);
 		TEST_CHECK(e.list_size() == 2);
@@ -265,7 +285,7 @@ TORRENT_TEST(lazy_entry)
 		TEST_CHECK(e.list_at(1)->string_value() == std::string("aaa"));
 		TEST_CHECK(e.list_at(1)->string_length() == 3);
 		section = e.list_at(1)->data_section();
-		TEST_CHECK(std::memcmp("3:aaa", section.first, section.second) == 0);
+		TEST_CHECK(std::memcmp("3:aaa", section.first, std::size_t(section.second)) == 0);
 		TEST_CHECK(section.second == 5);
 	}
 
@@ -275,9 +295,9 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret == 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 		std::pair<const char*, int> section = e.data_section();
-		TEST_CHECK(std::memcmp(b, section.first, section.second) == 0);
+		TEST_CHECK(std::memcmp(b, section.first, std::size_t(section.second)) == 0);
 		TEST_CHECK(section.second == sizeof(b) - 1);
 		TEST_CHECK(e.type() == lazy_entry::dict_t);
 		TEST_CHECK(e.dict_size() == 4);
@@ -300,7 +320,7 @@ TORRENT_TEST(lazy_entry)
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret == 0);
 		TEST_CHECK(e.dict_size() == 1);
-		lazy_entry* d = e.dict_find(std::string("a\0b", 3));
+		lazy_entry* d = e.dict_find({"a\0b", 3});
 		TEST_CHECK(d);
 		TEST_EQUAL(d->type(), lazy_entry::int_t);
 		TEST_EQUAL(d->int_value(), 1);
@@ -313,9 +333,8 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_CHECK(ec == error_code(bdecode_errors::expected_value
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::expected_value));
 	}
 
 	// test strings with overflow length-prefix
@@ -325,9 +344,8 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_CHECK(ec == error_code(bdecode_errors::overflow
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::overflow));
 	}
 
 	// test integers that don't fit in 64 bits
@@ -337,7 +355,7 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret == 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 		// the lazy aspect makes this overflow when asking for
 		// the value. turning it to zero.
 		TEST_CHECK(e.int_value() == 0);
@@ -350,7 +368,7 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret == 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 		TEST_CHECK(e.int_value() == 9223372036854775807LL);
 	}
 
@@ -361,7 +379,7 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec);
 		TEST_CHECK(ret == 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 		TEST_CHECK(e.int_value() == -9223372036854775807LL);
 	}
 
@@ -382,10 +400,10 @@ TORRENT_TEST(lazy_entry)
 			, 0xa1, 0x88, 0x7a, 0x8d, 0xc3, 0xd6, 0x31, 0x3a
 			, 0x79, 0x31, 0xae, 0x71, 0x65, 0};
 
-		printf("%s\n", buf);
+		std::printf("%s\n", buf);
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode((char*)buf, (char*)buf + sizeof(buf), e, ec);
+		int ret = lazy_bdecode(reinterpret_cast<char*>(buf), reinterpret_cast<char*>(buf) + sizeof(buf), e, ec);
 		TEST_CHECK(ret == -1);
 	}
 
@@ -404,8 +422,7 @@ TORRENT_TEST(lazy_entry)
 		error_code ec;
 		int ret = lazy_bdecode(b, b + sizeof(b), e, ec);
 		TEST_CHECK(ret != 0);
-		TEST_EQUAL(ec, error_code(bdecode_errors::depth_exceeded
-			, get_bdecode_category()));
+		TEST_EQUAL(ec, error_code(bdecode_errors::depth_exceeded));
 	}
 
 	// test the item limit
@@ -419,10 +436,9 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + i + 1, e, ec, NULL, 1000, 1000);
+		int ret = lazy_bdecode(b, b + i + 1, e, ec, nullptr, 1000, 1000);
 		TEST_CHECK(ret != 0);
-		TEST_EQUAL(ec, error_code(bdecode_errors::limit_exceeded
-			, get_bdecode_category()));
+		TEST_EQUAL(ec, error_code(bdecode_errors::limit_exceeded));
 	}
 
 	// test unexpected EOF
@@ -431,11 +447,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof));
 	}
 
 	// test unexpected EOF (really expected terminator)
@@ -444,11 +459,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof));
 	}
 
 	// test expected string
@@ -458,11 +472,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::expected_digit
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::expected_digit));
 	}
 
 	// test unexpected EOF while parsing dict key
@@ -471,11 +484,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof));
 	}
 
 	// test unexpected EOF while parsing dict key
@@ -484,11 +496,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof));
 	}
 
 	// test expected string while parsing dict key
@@ -497,11 +508,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::expected_digit
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::expected_digit));
 	}
 
 	// test unexpected EOF while parsing int
@@ -510,11 +520,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof));
 	}
 
 	// test unexpected EOF while parsing int
@@ -523,11 +532,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof));
 	}
 
 
@@ -537,11 +545,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::expected_colon
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::expected_colon));
 	}
 
 	// test empty string
@@ -550,11 +557,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_EQUAL(ret, -1);
-		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof
-			, get_bdecode_category()));
-		printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof));
+		std::printf("%s\n", print_entry(e).c_str());
 	}
 
 	// test partial string
@@ -563,11 +569,10 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_CHECK(ret != 0);
-		printf("%s\n", print_entry(e).c_str());
-		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof
-			, get_bdecode_category()));
+		std::printf("%s\n", print_entry(e).c_str());
+		TEST_EQUAL(ec, error_code(bdecode_errors::unexpected_eof));
 	}
 
 	// test pascal string dict
@@ -576,16 +581,16 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_EQUAL(ret, 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 
 		pascal_string ps = e.dict_find_pstr("foobar");
-		TEST_EQUAL(memcmp(ps.ptr, "barfoo", ps.len), 0);
+		TEST_EQUAL(std::memcmp(ps.ptr, "barfoo", std::size_t(ps.len)), 0);
 		TEST_EQUAL(ps.len, 6);
 
 		ps = e.dict_find_pstr("foobar2");
-		TEST_EQUAL(ps.ptr, static_cast<char const*>(0));
+		TEST_EQUAL(ps.ptr, static_cast<char const*>(nullptr));
 		TEST_EQUAL(ps.len, 0);
 	}
 
@@ -595,23 +600,24 @@ TORRENT_TEST(lazy_entry)
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, NULL);
+		int ret = lazy_bdecode(b, b + sizeof(b)-1, e, ec, nullptr);
 		TEST_EQUAL(ret, 0);
-		printf("%s\n", print_entry(e).c_str());
+		std::printf("%s\n", print_entry(e).c_str());
 
 		TEST_EQUAL(e.list_size(), 2);
 		pascal_string ps = e.list_pstr_at(0);
-		TEST_EQUAL(memcmp(ps.ptr, "foobar", ps.len), 0);
+		TEST_EQUAL(std::memcmp(ps.ptr, "foobar", std::size_t(ps.len)), 0);
 		TEST_EQUAL(ps.len, 6);
 
 		ps = e.list_pstr_at(1);
-		TEST_EQUAL(ps.ptr, static_cast<char const*>(0));
+		TEST_EQUAL(ps.ptr, static_cast<char const*>(nullptr));
 		TEST_EQUAL(ps.len, 0);
 	}
 
 	{
 		unsigned char buf[] = { 0x44, 0x91, 0x3a };
-		entry ent = bdecode(buf, buf + sizeof(buf));
+		error_code ec;
+		entry ent = bdecode({reinterpret_cast<char*>(buf), int(sizeof(buf))}, ec);
 		TEST_CHECK(ent == entry());
 	}
 
@@ -621,14 +627,14 @@ TORRENT_TEST(lazy_entry)
 		for (int i = 0; i < 1000; ++i)
 		{
 			char tmp[20];
-			snprintf(tmp, sizeof(tmp), "i%de", i);
+			std::snprintf(tmp, sizeof(tmp), "i%de", i);
 			buf += tmp;
 		}
 		buf += "e";
 
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode((char*)&buf[0], (char*)&buf[0] + buf.size(), e, ec);
+		int ret = lazy_bdecode(buf.data(), buf.data() + buf.size(), e, ec);
 		TEST_EQUAL(ret, 0);
 		TEST_EQUAL(e.type(), lazy_entry::list_t);
 		TEST_EQUAL(e.list_size(), 1000);
@@ -644,22 +650,22 @@ TORRENT_TEST(lazy_entry)
 		for (int i = 0; i < 1000; ++i)
 		{
 			char tmp[30];
-			snprintf(tmp, sizeof(tmp), "4:%04di%de", i, i);
+			std::snprintf(tmp, sizeof(tmp), "4:%04di%de", i, i);
 			buf += tmp;
 		}
 		buf += "e";
 
-		printf("%s\n", buf.c_str());
+		std::printf("%s\n", buf.c_str());
 		lazy_entry e;
 		error_code ec;
-		int ret = lazy_bdecode((char*)&buf[0], (char*)&buf[0] + buf.size(), e, ec);
+		int ret = lazy_bdecode(buf.data(), buf.data() + buf.size(), e, ec);
 		TEST_EQUAL(ret, 0);
 		TEST_EQUAL(e.type(), lazy_entry::dict_t);
 		TEST_EQUAL(e.dict_size(), 1000);
 		for (int i = 0; i < 1000; ++i)
 		{
 			char tmp[30];
-			snprintf(tmp, sizeof(tmp), "%04d", i);
+			std::snprintf(tmp, sizeof(tmp), "%04d", i);
 			TEST_EQUAL(e.dict_find_int_value(tmp), i);
 		}
 	}
@@ -667,7 +673,7 @@ TORRENT_TEST(lazy_entry)
 	// test parse_int
 	{
 		char b[] = "1234567890e";
-		boost::int64_t val = 0;
+		std::int64_t val = 0;
 		bdecode_errors::error_code_enum ec = bdecode_errors::no_error;
 		char const* e = parse_int(b, b + sizeof(b)-1, 'e', val, ec);
 		TEST_CHECK(ec == bdecode_errors::no_error);
@@ -678,7 +684,7 @@ TORRENT_TEST(lazy_entry)
 	// test invalid digit
 	{
 		char b[] = "0o";
-		boost::int64_t val = 0;
+		std::int64_t val = 0;
 		bdecode_errors::error_code_enum ec;
 		char const* e = parse_int(b, b + sizeof(b)-1, 'e', val, ec);
 		TEST_EQUAL(ec, bdecode_errors::expected_digit);
@@ -687,7 +693,7 @@ TORRENT_TEST(lazy_entry)
 
 	{
 		char b[] = "9223372036854775808:";
-		boost::int64_t val = 0;
+		std::int64_t val = 0;
 		bdecode_errors::error_code_enum ec;
 		char const* e = parse_int(b, b + sizeof(b)-1, ':', val, ec);
 		TEST_CHECK(ec == bdecode_errors::overflow);
@@ -696,7 +702,7 @@ TORRENT_TEST(lazy_entry)
 
 	{
 		char b[] = "928";
-		boost::int64_t val = 0;
+		std::int64_t val = 0;
 		bdecode_errors::error_code_enum ec = bdecode_errors::no_error;
 		char const* e = parse_int(b, b + sizeof(b)-1, ':', val, ec);
 		TEST_CHECK(ec == bdecode_errors::no_error);
@@ -714,14 +720,18 @@ TORRENT_TEST(lazy_entry)
 
 		for (int i = 0; i < int(sizeof(b)/sizeof(b[0])); ++i)
 		{
-			lazy_entry e;
+			lazy_entry tmp;
 			error_code ec;
-			int ret = lazy_bdecode(b[i], b[i] + strlen(b[i]), e, ec, NULL);
+			int ret = lazy_bdecode(b[i], b[i] + strlen(b[i]), tmp, ec, nullptr);
+			lazy_entry e;
+			e = std::move(tmp);
 			TEST_EQUAL(ret, -1);
 			TEST_CHECK(ec == error_code(bdecode_errors::unexpected_eof));
-			printf("%s\n", print_entry(e).c_str());
+			std::printf("%s\n", print_entry(e).c_str());
+
+			lazy_entry* moved = new lazy_entry(std::move(e));
+			delete moved;
 		}
 	}
 }
-#endif // TORRENT_NO_DEPRECATE
-
+#endif // TORRENT_ABI_VERSION
